/*
  Q Light Controller
  vcwidgetproperties.cpp

  Copyright (c) Heikki Junnila

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  Version 2 as published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details. The license is
  in the file "COPYING".

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <QWidget>
#include <QtXml>

#include "vcwidgetproperties.h"

VCWidgetProperties::VCWidgetProperties()
{
    m_state = Qt::WindowNoState;
    m_visible = false;
    m_x = 100;
    m_y = 100;
    m_width = 0;
    m_height = 0;
}

VCWidgetProperties::VCWidgetProperties(const VCWidgetProperties& properties)
{
    m_state = properties.m_state;
    m_visible = properties.m_visible;
    m_x = properties.m_x;
    m_y = properties.m_y;
    m_width = properties.m_width;
    m_height = properties.m_height;
}

VCWidgetProperties::~VCWidgetProperties()
{
}

QFlags <Qt::WindowState> VCWidgetProperties::state() const
{
    return m_state;
}

bool VCWidgetProperties::visible() const
{
    return m_visible;
}

int VCWidgetProperties::x() const
{
    return m_x;
}

int VCWidgetProperties::y() const
{
    return m_y;
}

int VCWidgetProperties::width() const
{
    return m_width;
}

int VCWidgetProperties::height() const
{
    return m_height;
}

void VCWidgetProperties::store(QWidget* widget)
{
    Q_ASSERT(widget != NULL);
    m_state = widget->windowState();
    m_visible = widget->isVisible();
    m_x = widget->x();
    m_y = widget->y();
    m_width = widget->width();
    m_height = widget->height();
}

bool VCWidgetProperties::loadXML(const QDomElement& root)
{
    if (root.tagName() != KXMLQLCWidgetProperties)
    {
        qWarning() << Q_FUNC_INFO << "Widget Properties node not found";
        return false;
    }

    QDomNode node = root.firstChild();
    while (node.isNull() == false)
    {
        QDomElement tag = node.toElement();

        if (tag.tagName() == KXMLQLCWidgetX)
            m_x = tag.text().toInt();
        else if (tag.tagName() == KXMLQLCWidgetY)
            m_y = tag.text().toInt();
        else if (tag.tagName() == KXMLQLCWidgetWidth)
            m_width = tag.text().toInt();
        else if (tag.tagName() == KXMLQLCWidgetHeight)
            m_height = tag.text().toInt();
        else if (tag.tagName() == KXMLQLCWidgetState)
            m_state = Qt::WindowState(tag.text().toInt());
        else if (tag.tagName() == KXMLQLCWidgetVisible)
            m_visible = bool(tag.text().toInt());
        else
            qWarning() << Q_FUNC_INFO << "Unknown widget tag:" << tag.tagName();

        node = node.nextSibling();
    }

    return true;
}

bool VCWidgetProperties::saveXML(QDomDocument* doc, QDomElement* root)
{
    QDomElement prop_root;
    QDomElement tag;
    QDomText text;
    QString str;

    Q_ASSERT(doc != NULL);
    Q_ASSERT(root != NULL);

    /* Widget Properties entry */
    prop_root = doc->createElement(KXMLQLCWidgetProperties);
    root->appendChild(prop_root);

    /* X */
    tag = doc->createElement(KXMLQLCWidgetX);
    prop_root.appendChild(tag);
    text = doc->createTextNode(QString("%1").arg(m_x));
    tag.appendChild(text);

    /* Y */
    tag = doc->createElement(KXMLQLCWidgetY);
    prop_root.appendChild(tag);
    text = doc->createTextNode(QString("%1").arg(m_y));
    tag.appendChild(text);

    /* W */
    tag = doc->createElement(KXMLQLCWidgetWidth);
    prop_root.appendChild(tag);
    text = doc->createTextNode(QString("%1").arg(m_width));
    tag.appendChild(text);

    /* H */
    tag = doc->createElement(KXMLQLCWidgetHeight);
    prop_root.appendChild(tag);
    text = doc->createTextNode(QString("%1").arg(m_height));
    tag.appendChild(text);

    /* Window state */
    tag = doc->createElement(KXMLQLCWidgetState);
    prop_root.appendChild(tag);
    text = doc->createTextNode(QString("%1").arg(m_state));
    tag.appendChild(text);

    /* Visible state */
    tag = doc->createElement(KXMLQLCWidgetVisible);
    prop_root.appendChild(tag);
    text = doc->createTextNode(QString("%1").arg(m_visible));
    tag.appendChild(text);

    return true;
}

